@@ -32,7 +32,14 @@ public:
 
 private Q_SLOTS:
     void shouldExportVcard4();
+    void shouldExportVcard4WithoutTime();
     void shouldExportVcard3();
+    void shouldExportVcard3WithoutTime();
+    void shouldParseBirthDay();
+    void shouldParseBirthDayWithoutTime();
+    void shouldParseBirthDayWithoutTimeAndYear();
+    void shouldExportVcard4WithoutTimeAndWithoutYear();
+    void shouldExportVcard3WithoutTimeAndWithoutYear();
 };
 
 #endif // BIRTHDAYTEST_H
