@@ -32,6 +32,7 @@
 #include <QtCore/QString>
 #include <QtCore/QBuffer>
 #include <QDebug>
+#include <QRegularExpression>
 
 using namespace KContacts;
 
@@ -219,7 +220,9 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         }
 
         // BDAY
-        card.addLine(VCardLine(QStringLiteral("BDAY"), createDateTime((*addrIt).birthday(), version)));
+        const bool withTime = (*addrIt).birthdayHasTime();
+        const QString birthdayString = createDateTime((*addrIt).birthday(), version, withTime);
+        card.addLine(VCardLine(QStringLiteral("BDAY"), birthdayString));
 
         //Laurent: 31 Jan 2015. Not necessary to export it. When Categories were changes as AkonadiTag nobody thought that it was break categorie support...
         //=> not necessary to export just tag...
@@ -276,6 +279,17 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
                 card.addLine(line);
             }
         }
+        // CLIENTPIDMAP
+        if (version == VCard::v4_0) {
+            const ClientPidMap::List clientpidmapList = (*addrIt).clientPidMapList();
+            ClientPidMap::List::ConstIterator clientPidMapIt;
+            ClientPidMap::List::ConstIterator clientPidMapEnd(clientpidmapList.end());
+            for (clientPidMapIt = clientpidmapList.begin(); clientPidMapIt != clientPidMapEnd; ++clientPidMapIt) {
+                VCardLine line(QStringLiteral("CLIENTPIDMAP"), (*clientPidMapIt).clientPidMap());
+                addParameters(line, (*clientPidMapIt).parameters());
+                card.addLine(line);
+            }
+        }
         // EMAIL
         const Email::List emailList = (*addrIt).emailList();
         Email::List::ConstIterator emailIt;
@@ -650,7 +664,7 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
                     const QDate date = QDate::fromString(value, Qt::ISODate);
                     QDateTime dt = QDateTime(date);
                     dt.setTime(QTime());
-                    VCardLine line(QStringLiteral("ANNIVERSARY"), createDateTime(dt, version));
+                    VCardLine line(QStringLiteral("ANNIVERSARY"), createDateTime(dt, version, false));
                     card.addLine(line);
                 }
             } else if (identifier.toLower() == QLatin1String("x-kaddressbook-x-spousesname") && version == VCard::v4_0) {
@@ -775,7 +789,9 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                 }
                 // BDAY
                 else if (identifier == QLatin1String("bday")) {
-                    addr.setBirthday(parseDateTime((*lineIt).value().toString()));
+                    bool withTime;
+                    const QDateTime bday = parseDateTime((*lineIt).value().toString(), &withTime);
+                    addr.setBirthday(bday, withTime);
                 }
 
                 // CATEGORIES
@@ -1117,6 +1133,13 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                     related.setParameters((*lineIt).parameterMap());
                     addr.insertRelationShip(related);
                 }
+                // CLIENTPIDMAP (vcard 4.0)
+                else if (identifier == QLatin1String("clientpidmap")) {
+                    ClientPidMap clientpidmap;
+                    clientpidmap.setClientPidMap((*lineIt).value().toString());
+                    clientpidmap.setParameters((*lineIt).parameterMap());
+                    addr.insertClientPidMap(clientpidmap);
+                }
                 // X-
                 //TODO import X-GENDER
                 else if (identifier.startsWith(QLatin1String("x-"))) {
@@ -1180,95 +1203,122 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
     return addrList;
 }
 
-QDateTime VCardTool::parseDateTime(const QString &str) const
+QDateTime VCardTool::parseDateTime(const QString &str, bool *timeValid)
 {
-    QDate date;
-    QTime time;
+    const QStringList strings = str.split(QLatin1Char('T'));
 
-    if (str.indexOf(QLatin1Char('-')) == -1) {       // is base format (yyyymmdd)
-        date = QDate(str.leftRef(4).toInt(), str.midRef(4, 2).toInt(),
-                     str.midRef(6, 2).toInt());
-    } else { // is extended format yyyy-mm-dd
-        date = QDate(str.leftRef(4).toInt(), str.midRef(5, 2).toInt(),
-                     str.midRef(8, 2).toInt());
+    QString dateString = strings.at(0);
+    const bool noYear = dateString.startsWith(QLatin1String("--"));
+    dateString = dateString.replace(QLatin1String("-"), QString());
+    QDate date = QDate::fromString(dateString, QStringLiteral("yyyyMMdd"));
+    if (noYear) {
+        date = QDate::fromString(dateString, QStringLiteral("MMdd"));
+        date = date.addYears(-1900);
     }
 
-    // does it also contain a time ? (Note: mm, ss are optional according ISO-8601)
-    int timeStart = str.indexOf(QLatin1Char('T'));
-    if (timeStart >= 0) {
-        int hour = 0, minute = 0, second = 0;
-
-        hour = str.midRef(timeStart + 1, 2).toInt();    // hour must always be given
-
-        if (str.indexOf(QLatin1Char(':'), timeStart + 1) > 0) {        // extended format (hh:mm:ss)
-            if (str.length() >= (timeStart + 5)) {
-                minute = str.midRef(timeStart + 4, 2).toInt();
-                if (str.length() >= (timeStart + 8)) {
-                    second = str.midRef(timeStart + 7, 2).toInt();
+    QTime time;
+    Qt::TimeSpec spec = Qt::LocalTime;
+    int offsetSecs = 0;
+    if (strings.length() > 1) {
+        QString timeString = strings.at(1);
+        timeString = timeString.replace(QLatin1String(":"), QString());
+        QStringList timeStrings = timeString.split(QRegularExpression(QStringLiteral("[Z+-]")));
+        const QString hhmmssString = timeStrings.at(0);
+        switch (hhmmssString.size()) {
+        case 2:
+            time = QTime::fromString(hhmmssString, QStringLiteral("hh"));
+            break;
+        case 4:
+            time = QTime::fromString(hhmmssString, QStringLiteral("hhmm"));
+            break;
+        case 6:
+            time = QTime::fromString(hhmmssString, QStringLiteral("hhmmss"));
+            break;
+        }
+        if (timeStrings.length() > 1) {
+            if (timeString.contains(QLatin1Char('Z'))) {
+                spec = Qt::UTC;
+            } else {
+                spec = Qt::OffsetFromUTC;
+                QTime offsetTime;
+                const QString offsetString = timeStrings.at(1);
+                switch (offsetString.size()) {
+                case 2:
+                    offsetTime = QTime::fromString(offsetString, QStringLiteral("hh"));
+                    break;
+                case 4:
+                    offsetTime = QTime::fromString(offsetString, QStringLiteral("hhmm"));
+                    break;
                 }
+                offsetSecs = offsetTime.hour() * 3600 + offsetTime.minute() * 60;
             }
-        } else {  // basic format (hhmmss)
-            if (str.length() >= (timeStart + 4)) {
-                minute = str.midRef(timeStart + 3, 2).toInt();
-                if (str.length() >= (timeStart + 6)) {
-                    second = str.midRef(timeStart + 5, 2).toInt();
-                }
+            if (timeString.contains(QLatin1Char('-'))) {
+                offsetSecs *= -1;
             }
         }
-
-        time = QTime(hour, minute, second);
+    }
+    if (timeValid) {
+        *timeValid = time.isValid();
     }
 
-    Qt::TimeSpec spec = (str.right(1) == QLatin1String("Z")) ? Qt::UTC : Qt::LocalTime;
-
-    QDateTime dateTime(date);
-
-    if (time.isValid()) {
-        dateTime.setTime(time);
-    }
-
-    dateTime.setTimeSpec(spec);
-    return dateTime;
+    return QDateTime(date, time, spec, offsetSecs);
 }
 
-QString VCardTool::createDateTime(const QDateTime &dateTime, VCard::Version version) const
+QString VCardTool::createDateTime(const QDateTime &dateTime, VCard::Version version, bool withTime)
 {
-    QString str;
-    if (version == VCard::v4_0) {
-        if (dateTime.date().isValid()) {
-            str.sprintf("%4d%02d%02d", dateTime.date().year(), dateTime.date().month(),
-                        dateTime.date().day());
-            if (dateTime.time().isValid()) {
-                QString tmp;
-                tmp.sprintf("T%02d%02d%02d", dateTime.time().hour(), dateTime.time().minute(),
-                            dateTime.time().second());
-                str += tmp;
-
-                if (dateTime.timeSpec() == Qt::UTC) {
-                    str += QLatin1Char('Z');
-                }
-            }
+    if (!dateTime.date().isValid()) {
+        return QString();
+    }
+    QString str = createDate(dateTime.date(), version);
+    if (!withTime) {
+        return str;
+    }
+    str += createTime(dateTime.time(), version);
+    if (dateTime.timeSpec() == Qt::UTC) {
+        str += QLatin1Char('Z');
+    } else if (dateTime.timeSpec() == Qt::OffsetFromUTC) {
+        const int offsetSecs = dateTime.offsetFromUtc();
+        if (offsetSecs >= 0) {
+            str += QLatin1Char('+');
+        } else {
+            str += QLatin1Char('-');
         }
-
-    } else {
-        if (dateTime.date().isValid()) {
-            str.sprintf("%4d-%02d-%02d", dateTime.date().year(), dateTime.date().month(),
-                        dateTime.date().day());
-            if (dateTime.time().isValid()) {
-                QString tmp;
-                tmp.sprintf("T%02d:%02d:%02d", dateTime.time().hour(), dateTime.time().minute(),
-                            dateTime.time().second());
-                str += tmp;
-
-                if (dateTime.timeSpec() == Qt::UTC) {
-                    str += QLatin1Char('Z');
-                }
-            }
+        QTime offsetTime = QTime(0, 0).addSecs(abs(offsetSecs));
+        if (version == VCard::v4_0) {
+            str += offsetTime.toString(QStringLiteral("HHmm"));
+        } else {
+            str += offsetTime.toString(QStringLiteral("HH:mm"));
         }
     }
     return str;
 }
 
+QString VCardTool::createDate(const QDate &date, VCard::Version version)
+{
+    QString format;
+    if (date.year() > 0) {
+        format = QStringLiteral("yyyyMMdd");
+    } else {
+        format = QStringLiteral("--MMdd");
+    }
+    if (version != VCard::v4_0) {
+        format = format.replace(QStringLiteral("yyyy"), QStringLiteral("yyyy-"));
+        format = format.replace(QStringLiteral("MM"), QStringLiteral("MM-"));
+    }
+    return date.toString(format);
+}
+
+QString VCardTool::createTime(const QTime &time, VCard::Version version)
+{
+    QString format;
+    if (version == VCard::v4_0) {
+        format = QStringLiteral("HHmmss");
+    } else {
+        format = QStringLiteral("HH:mm:ss");
+    }
+    return QStringLiteral("T") + time.toString(format);
+}
+
 Picture VCardTool::parsePicture(const VCardLine &line) const
 {
     Picture pic;
