@@ -58,6 +58,11 @@ public:
      */
     Addressee::List parseVCards(const QByteArray &vcard) const;
 
+    static QDateTime parseDateTime(const QString &str, bool *timeValid = 0);
+    static QString createDateTime(const QDateTime &dateTime, VCard::Version version, bool withTime = true);
+    static QString createDate(const QDate &date, VCard::Version version);
+    static QString createTime(const QTime &time, VCard::Version version);
+
 private:
 
     QByteArray createVCards(const Addressee::List &list,
@@ -69,9 +74,6 @@ private:
      */
     QStringList splitString(QChar sep, const QString &value) const;
 
-    QDateTime parseDateTime(const QString &str) const;
-    QString createDateTime(const QDateTime &dateTime, VCard::Version version) const;
-
     Picture parsePicture(const VCardLine &line) const;
     VCardLine createPicture(const QString &identifier, const Picture &pic, VCard::Version version) const;
 
